@@ -16,7 +16,19 @@ Usage example (Python 2.4 or newer)::
         if n < 2: return 1
         else: return n * fn(n-1)
 
-    print fn(42)
+    print(fn(42))
+
+Or without imports, with some hack
+
+    $ python -m profilehooks yourmodule
+
+    @profile    # or @coverage
+    def fn(n):
+        if n < 2: return 1
+        else: return n * fn(n-1)
+
+    print(fn(42))
+
 
 Usage example (Python 2.3 or older)::
 
@@ -62,7 +74,7 @@ Caveats
   executed.  For this reason coverage analysis now uses trace.py which is
   slower, but more accurate.
 
-Copyright (c) 2004--2008 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2004--2012 Marius Gedminas <marius@pov.lt>
 Copyright (c) 2007 Hanno Schlichting
 Copyright (c) 2008 Florian Schulze
 
@@ -88,13 +100,12 @@ Released under the MIT licence since December 2006:
 
 (Previously it was distributed under the GNU General Public Licence.)
 """
-# $Id: profilehooks.py 26 2009-03-31 17:46:08Z mg $
 
 __author__ = "Marius Gedminas (marius@gedmin.as)"
-__copyright__ = "Copyright 2004-2009 Marius Gedminas"
+__copyright__ = "Copyright 2004-2013 Marius Gedminas"
 __license__ = "MIT"
-__version__ = "1.4"
-__date__ = "2009-03-31"
+__version__ = "1.7"
+__date__ = "2013-10-16"
 
 
 import atexit
@@ -134,6 +145,8 @@ import time
 # registry of available profilers
 AVAILABLE_PROFILERS = {}
 
+__all__ = ['coverage', 'coverage_with_hotshot', 'profile', 'timecall']
+
 
 def profile(fn=None, skip=0, filename=None, immediate=False, dirs=False,
             sort=None, entries=40,
@@ -192,7 +205,7 @@ def profile(fn=None, skip=0, filename=None, immediate=False, dirs=False,
             ...
 
     """
-    if fn is None: # @profile() syntax -- we are a decorator maker
+    if fn is None:  # @profile() syntax -- we are a decorator maker
         def decorator(fn):
             return profile(fn, skip=skip, filename=filename,
                            immediate=immediate, dirs=dirs,
@@ -216,6 +229,7 @@ def profile(fn=None, skip=0, filename=None, immediate=False, dirs=False,
          # or HotShotFuncProfile
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
+
     def new_fn(*args, **kw):
         return fp(*args, **kw)
     new_fn.__doc__ = fn.__doc__
@@ -244,9 +258,10 @@ def coverage(fn):
             ...
 
     """
-    fp = TraceFuncCoverage(fn) # or HotShotFuncCoverage
+    fp = TraceFuncCoverage(fn)  # or HotShotFuncCoverage
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
+
     def new_fn(*args, **kw):
         return fp(*args, **kw)
     new_fn.__doc__ = fn.__doc__
@@ -268,6 +283,7 @@ def coverage_with_hotshot(fn):
     fp = HotShotFuncCoverage(fn)
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
+
     def new_fn(*args, **kw):
         return fp(*args, **kw)
     new_fn.__doc__ = fn.__doc__
@@ -333,17 +349,17 @@ class FuncProfile(object):
     def print_stats(self):
         """Print profile information to sys.stdout."""
         funcname = self.fn.__name__
-        filename = self.fn.func_code.co_filename
-        lineno = self.fn.func_code.co_firstlineno
-        print
-        print "*** PROFILER RESULTS ***"
-        print "%s (%s:%s)" % (funcname, filename, lineno)
-        print "function called %d times" % self.ncalls,
+        filename = self.fn.__code__.co_filename
+        lineno = self.fn.__code__.co_firstlineno
+        print("")
+        print("*** PROFILER RESULTS ***")
+        print("%s (%s:%s)" % (funcname, filename, lineno))
         if self.skipped:
-            print "(%d calls not profiled)" % self.skipped
+            skipped = "(%d calls not profiled)" % self.skipped
         else:
-            print
-        print
+            skipped = ""
+        print("function called %d times%s" % (self.ncalls, skipped))
+        print("")
         stats = self.stats
         if self.filename:
             stats.dump_stats(self.filename)
@@ -436,17 +452,17 @@ if hotshot is not None:
             """
             self.profiler.close()
             funcname = self.fn.__name__
-            filename = self.fn.func_code.co_filename
-            lineno = self.fn.func_code.co_firstlineno
-            print
-            print "*** PROFILER RESULTS ***"
-            print "%s (%s:%s)" % (funcname, filename, lineno)
-            print "function called %d times" % self.ncalls,
+            filename = self.fn.__code__.co_filename
+            lineno = self.fn.__code__.co_firstlineno
+            print("")
+            print("*** PROFILER RESULTS ***")
+            print("%s (%s:%s)" % (funcname, filename, lineno))
             if self.skipped:
-                print "(%d calls not profiled)" % self.skipped
+                skipped = "(%d calls not profiled)" % self.skipped
             else:
-                print
-            print
+                skipped = ""
+            print("function called %d times%s" % (self.ncalls, skipped))
+            print("")
             stats = hotshot.stats.load(self.logfilename)
             # hotshot.stats.load takes ages, and the .prof file eats megabytes, but
             # a saved stats object is small and fast
@@ -459,7 +475,6 @@ if hotshot is not None:
 
     AVAILABLE_PROFILERS['hotshot'] = HotShotFuncProfile
 
-
     class HotShotFuncCoverage:
         """Coverage analysis for a function (uses _hotshot).
 
@@ -499,13 +514,13 @@ if hotshot is not None:
             """
             self.profiler.close()
             funcname = self.fn.__name__
-            filename = self.fn.func_code.co_filename
-            lineno = self.fn.func_code.co_firstlineno
-            print
-            print "*** COVERAGE RESULTS ***"
-            print "%s (%s:%s)" % (funcname, filename, lineno)
-            print "function called %d times" % self.ncalls
-            print
+            filename = self.fn.__code__.co_filename
+            lineno = self.fn.__code__.co_firstlineno
+            print("")
+            print("*** COVERAGE RESULTS ***")
+            print("%s (%s:%s)" % (funcname, filename, lineno))
+            print("function called %d times" % self.ncalls)
+            print("")
             fs = FuncSource(self.fn)
             reader = hotshot.log.LogReader(self.logfilename)
             for what, (filename, lineno, funcname), tdelta in reader:
@@ -522,7 +537,7 @@ if hotshot is not None:
                         lineno = fs.firstcodelineno
                     fs.mark(lineno)
             reader.close()
-            print fs
+            print(fs)
 
 
 class TraceFuncCoverage:
@@ -573,22 +588,22 @@ class TraceFuncCoverage:
         This function is registered as an atexit hook.
         """
         funcname = self.fn.__name__
-        filename = self.fn.func_code.co_filename
-        lineno = self.fn.func_code.co_firstlineno
-        print
-        print "*** COVERAGE RESULTS ***"
-        print "%s (%s:%s)" % (funcname, filename, lineno)
-        print "function called %d times" % self.ncalls
-        print
+        filename = self.fn.__code__.co_filename
+        lineno = self.fn.__code__.co_firstlineno
+        print("")
+        print("*** COVERAGE RESULTS ***")
+        print("%s (%s:%s)" % (funcname, filename, lineno))
+        print("function called %d times" % self.ncalls)
+        print("")
         fs = FuncSource(self.fn)
         for (filename, lineno), count in self.tracer.counts.items():
             if filename != fs.filename:
                 continue
             fs.mark(lineno, count)
-        print fs
+        print(fs)
         never_executed = fs.count_never_executed()
         if never_executed:
-            print "%d lines were not executed." % never_executed
+            print("%d lines were not executed." % never_executed)
 
 
 class FuncSource:
@@ -599,20 +614,27 @@ class FuncSource:
     def __init__(self, fn):
         self.fn = fn
         self.filename = inspect.getsourcefile(fn)
-        self.source, self.firstlineno = inspect.getsourcelines(fn)
         self.sourcelines = {}
-        self.firstcodelineno = self.firstlineno
-        self.find_source_lines()
+        self.source = []
+        self.firstlineno = self.firstcodelineno = 0
+        try:
+            self.source, self.firstlineno = inspect.getsourcelines(fn)
+            self.firstcodelineno = self.firstlineno
+            self.find_source_lines()
+        except IOError:
+            self.filename = None
 
     def find_source_lines(self):
         """Mark all executable source lines in fn as executed 0 times."""
+        if self.filename is None:
+            return
         strs = trace.find_strings(self.filename)
-        lines = trace.find_lines_from_code(self.fn.func_code, strs)
-        self.firstcodelineno = sys.maxint
+        lines = trace.find_lines_from_code(self.fn.__code__, strs)
         for lineno in lines:
-            self.firstcodelineno = min(self.firstcodelineno, lineno)
             self.sourcelines.setdefault(lineno, 0)
-        if self.firstcodelineno == sys.maxint:
+        if lines:
+            self.firstcodelineno = min(lines)
+        else:
             self.firstcodelineno = self.firstlineno
 
     def mark(self, lineno, count=1):
@@ -635,6 +657,8 @@ class FuncSource:
 
     def __str__(self):
         """Return annotated source code for the function."""
+        if self.filename is None:
+            return "cannot show coverage data since co_filename is None"
         lines = []
         lineno = self.firstlineno
         for line in self.source:
@@ -653,7 +677,7 @@ class FuncSource:
         return ''.join(lines)
 
 
-def timecall(fn=None, immediate=True):
+def timecall(fn=None, immediate=True, timer=time.time):
     """Wrap `fn` and print its execution time.
 
     Example::
@@ -669,15 +693,21 @@ def timecall(fn=None, immediate=True):
 
         @timecall(immediate=False)
 
+    You can also choose a timing method other than the default ``time.time()``,
+    e.g.:
+
+        @timecall(timer=time.clock)
+
     """
-    if fn is None: # @timecall() syntax -- we are a decorator maker
+    if fn is None:  # @timecall() syntax -- we are a decorator maker
         def decorator(fn):
-            return timecall(fn, immediate=immediate)
+            return timecall(fn, immediate=immediate, timer=timer)
         return decorator
     # @timecall syntax -- we are a decorator.
-    fp = FuncTimer(fn, immediate=immediate)
+    fp = FuncTimer(fn, immediate=immediate, timer=timer)
     # We cannot return fp or fp.__call__ directly as that would break method
     # definitions, instead we need to return a plain function.
+
     def new_fn(*args, **kw):
         return fp(*args, **kw)
     new_fn.__doc__ = fn.__doc__
@@ -689,38 +719,67 @@ def timecall(fn=None, immediate=True):
 
 class FuncTimer(object):
 
-    def __init__(self, fn, immediate):
+    def __init__(self, fn, immediate, timer):
         self.fn = fn
         self.ncalls = 0
         self.totaltime = 0
         self.immediate = immediate
+        self.timer = timer
         if not immediate:
             atexit.register(self.atexit)
 
     def __call__(self, *args, **kw):
         """Profile a singe call to the function."""
         fn = self.fn
+        timer = self.timer
         self.ncalls += 1
         try:
-            start = time.time()
+            start = timer()
             return fn(*args, **kw)
         finally:
-            duration = time.time() - start
+            duration = timer() - start
             self.totaltime += duration
             if self.immediate:
                 funcname = fn.__name__
-                filename = fn.func_code.co_filename
-                lineno = fn.func_code.co_firstlineno
-                print >> sys.stderr, "\n  %s (%s:%s):\n    %.3f seconds\n" % (
-                                        funcname, filename, lineno, duration)
+                filename = fn.__code__.co_filename
+                lineno = fn.__code__.co_firstlineno
+                sys.stderr.write("\n  %s (%s:%s):\n    %.3f seconds\n\n" % (
+                    funcname, filename, lineno, duration
+                ))
+                sys.stderr.flush()
+
     def atexit(self):
         if not self.ncalls:
             return
         funcname = self.fn.__name__
-        filename = self.fn.func_code.co_filename
-        lineno = self.fn.func_code.co_firstlineno
-        print ("\n  %s (%s:%s):\n"
-               "    %d calls, %.3f seconds (%.3f seconds per call)\n" % (
-                                funcname, filename, lineno, self.ncalls,
-                                self.totaltime, self.totaltime / self.ncalls))
+        filename = self.fn.__code__.co_filename
+        lineno = self.fn.__code__.co_firstlineno
+        print("\n  %s (%s:%s):\n"
+              "    %d calls, %.3f seconds (%.3f seconds per call)\n" % (
+                  funcname, filename, lineno, self.ncalls,
+                  self.totaltime, self.totaltime / self.ncalls)
+              )
 
+if __name__ == '__main__':
+
+    local = dict((name, globals()[name]) for name in __all__)
+    message = """********
+Injected `profilehooks`
+--------
+{}
+********
+""".format("\n".join(local.keys()))
+
+    def interact_():
+        from code import interact
+        interact(message, local=local)
+
+    def run_():
+        from runpy import run_module
+        print(message)
+        run_module(sys.argv[1], init_globals=local)
+
+    if len(sys.argv) == 1:
+        interact_()
+    else:
+        run_()
